@@ -19,8 +19,8 @@ if major == 2:
     # Specific to 3.x dev releases
     versions.append((3, 0))
 else:
-    minor_max = minor if minor != 0 and is_stable(minor) else minor + 1
-    for i in range(0, minor_max):
+    minor_max = minor if is_stable(minor) else minor + 1
+    for i in range(0, minor_max + 1):
         if is_stable(i):
             versions.append((3, i))
 
@@ -37,19 +37,32 @@ for version in versions:
  * Since: {major_version}.{minor_version}
  */
 #define SOUP_VERSION_{major_version}_{minor_version} (G_ENCODE_VERSION ({major_version}, {minor_version}))
-
 '''.format(major_version=version[0], minor_version=version[1])
 
-    version_attributes += '''#if SOUP_VERSION_MIN_REQUIRED >= SOUP_VERSION_{major_version}_{minor_version}
-# define SOUP_DEPRECATED_IN_{major_version}_{minor_version}                G_DEPRECATED
-# define SOUP_DEPRECATED_IN_{major_version}_{minor_version}_FOR(f)         G_DEPRECATED_FOR(f)
+    version_attributes += '''/**
+ * SOUP_DEPRECATED_IN_{major_version}_{minor_version}:
+ * A macro used to indicate a symbol was deprecated in this version.
+ */
+/**
+ * SOUP_DEPRECATED_IN_{major_version}_{minor_version}_FOR:
+ * @f: The recommended replacement function.
+ *
+ * A macro used to indicate a symbol was deprecated in this version with a replacement.
+ */
+#if SOUP_VERSION_MIN_REQUIRED >= SOUP_VERSION_{major_version}_{minor_version}
+# define SOUP_DEPRECATED_IN_{major_version}_{minor_version}                SOUP_DEPRECATED
+# define SOUP_DEPRECATED_IN_{major_version}_{minor_version}_FOR(f)         SOUP_DEPRECATED_FOR(f)
 #else
-# define SOUP_DEPRECATED_IN_{major_version}_{minor_version}
-# define SOUP_DEPRECATED_IN_{major_version}_{minor_version}_FOR(f)
+# define SOUP_DEPRECATED_IN_{major_version}_{minor_version}                _SOUP_EXTERN
+# define SOUP_DEPRECATED_IN_{major_version}_{minor_version}_FOR(f)         _SOUP_EXTERN
 #endif
 
+/**
+ * SOUP_AVAILABLE_IN_{major_version}_{minor_version}:
+ * A macro used to indicate a symbol is available in this version or later.
+ */
 #if SOUP_VERSION_MAX_ALLOWED < SOUP_VERSION_{major_version}_{minor_version}
-# define SOUP_AVAILABLE_IN_{major_version}_{minor_version}                 G_UNAVAILABLE({major_version}, {minor_version}) _SOUP_EXTERN
+# define SOUP_AVAILABLE_IN_{major_version}_{minor_version}                 SOUP_UNAVAILABLE({major_version}, {minor_version})
 #else
 # define SOUP_AVAILABLE_IN_{major_version}_{minor_version}                 _SOUP_EXTERN
 #endif
