@@ -653,9 +653,8 @@ soup_header_contains (const char *header, const char *token)
 }
 
 static void
-decode_quoted_string_inplace (GString *quoted_gstring)
+decode_quoted_string (char *quoted_string)
 {
-	char *quoted_string = quoted_gstring->str;
 	char *src, *dst;
 
 	src = quoted_string + 1;
@@ -669,11 +668,10 @@ decode_quoted_string_inplace (GString *quoted_gstring)
 }
 
 static gboolean
-decode_rfc5987_inplace (GString *encoded_gstring)
+decode_rfc5987 (char *encoded_string)
 {
 	char *q, *decoded;
 	gboolean iso_8859_1 = FALSE;
-	const char *encoded_string = encoded_gstring->str;
 
 	q = strchr (encoded_string, '\'');
 	if (!q)
@@ -705,7 +703,14 @@ decode_rfc5987_inplace (GString *encoded_gstring)
 		decoded = utf8;
 	}
 
-	g_string_assign (encoded_gstring, decoded);
+	/* If encoded_string was UTF-8, then each 3-character %-escape
+	 * will be converted to a single byte, and so decoded is
+	 * shorter than encoded_string. If encoded_string was
+	 * iso-8859-1, then each 3-character %-escape will be
+	 * converted into at most 2 bytes in UTF-8, and so it's still
+	 * shorter.
+	 */
+	strcpy (encoded_string, decoded);
 	g_free (decoded);
 	return TRUE;
 }
@@ -715,17 +720,15 @@ parse_param_list (const char *header, char delim, gboolean strict)
 {
 	GHashTable *params;
 	GSList *list, *iter;
+	char *item, *eq, *name_end, *value;
+	gboolean override, duplicated;
 
 	params = g_hash_table_new_full (soup_str_case_hash, 
 					soup_str_case_equal,
-					g_free, g_free);
+					g_free, NULL);
 
 	list = parse_list (header, delim);
 	for (iter = list; iter; iter = iter->next) {
-		char *item, *eq, *name_end;
-		gboolean override, duplicated;
-		GString *parsed_value = NULL;
-
 		item = iter->data;
 		override = FALSE;
 
@@ -740,19 +743,19 @@ parse_param_list (const char *header, char delim, gboolean strict)
 
 			*name_end = '\0';
 
-			parsed_value = g_string_new ((char *)skip_lws (eq + 1));
+			value = (char *)skip_lws (eq + 1);
 
 			if (name_end[-1] == '*' && name_end > item + 1) {
 				name_end[-1] = '\0';
-				if (!decode_rfc5987_inplace (parsed_value)) {
-					g_string_free (parsed_value, TRUE);
+				if (!decode_rfc5987 (value)) {
 					g_free (item);
 					continue;
 				}
 				override = TRUE;
-			} else if (parsed_value->str[0] == '"')
-				decode_quoted_string_inplace (parsed_value);
-		}
+			} else if (*value == '"')
+				decode_quoted_string (value);
+		} else
+			value = NULL;
 
 		duplicated = g_hash_table_lookup_extended (params, item, NULL, NULL);
 
@@ -760,16 +763,11 @@ parse_param_list (const char *header, char delim, gboolean strict)
 			soup_header_free_param_list (params);
 			params = NULL;
 			g_slist_foreach (iter, (GFunc)g_free, NULL);
-			if (parsed_value)
-				g_string_free (parsed_value, TRUE);
 			break;
-		} else if (override || !duplicated) {
-			g_hash_table_replace (params, item, parsed_value ? g_string_free (parsed_value, FALSE) : NULL);
-		} else {
-			if (parsed_value)
-				g_string_free (parsed_value, TRUE);
+		} else if (override || !duplicated)
+			g_hash_table_replace (params, item, value);
+		else
 			g_free (item);
-		}
 	}
 
 	g_slist_free (list);
@@ -904,7 +902,7 @@ append_param_rfc5987 (GString    *string,
 
 	g_string_append (string, name);
 	g_string_append (string, "*=UTF-8''");
-	encoded = g_uri_escape_string (value, "*'%()<>@,;:\\\"/[]?=", FALSE);
+	encoded = g_uri_escape_string (value, "!#$&+-.^_`|~", FALSE);
 	g_string_append (string, encoded);
 	g_free (encoded);
 }
@@ -914,7 +912,7 @@ append_param_quoted (GString    *string,
 		     const char *name,
 		     const char *value)
 {
-	gsize len;
+	int len;
 
 	g_string_append (string, name);
 	g_string_append (string, "=\"");
@@ -988,7 +986,7 @@ soup_header_g_string_append_param_quoted (GString    *string,
  * soup_header_g_string_append_param:
  * @string: a #GString being used to construct an HTTP header value
  * @name: a parameter name
- * @value: a parameter value, or %NULL
+ * @value: (nullable): a parameter value, or %NULL
  *
  * Appends something like `name=value` to @string, taking care to quote @value
  * if needed, and if so, to escape any quotes or backslashes in @value.
